#include <Fade_2D.h>
#include <stdlib.h>
#include <stdio.h>
#include <algorithm>
#include "someTools.h"

using namespace GEOM_FADE2D;

void createTestZones(Fade_2D& dt,Zone2*& pEarthZone,Zone2*& pMarsZone,Zone2*& pDiffZone);



int ex11_save_and_load_main()
{
	std::cout<<"* Example11: Save and Load - Store and reload triangulation data"<<std::endl;

	// * 1 *   Create 3 test zones
	Fade_2D dt;
	Zone2* pEarthZone(NULL);
	Zone2* pMarsZone(NULL);
	Zone2* pDiffZone(NULL);
	createTestZones(dt,pEarthZone,pMarsZone,pDiffZone);
	std::vector<Zone2*> vSaveZones;
	vSaveZones.push_back(pEarthZone);
	vSaveZones.push_back(pMarsZone);
	vSaveZones.push_back(pDiffZone);

	// * 2 *   Visualize
	pDiffZone->show("ex11_original_earth_minus_mars.ps",true,true);
	for(size_t i=0;i<vSaveZones.size();++i)
	{
		std::cout<<"Zone "<<i<<", numT="<<vSaveZones[i]->getNumberOfTriangles()<<std::endl;
	}


	// * 3 *   Save *all* triangles and zones
	std::cout<<"Saving using saveTriangulation()..."<<std::endl;
	dt.saveTriangulation("ex11_saveTriangulation.fade",vSaveZones);

	// * 4 *   Save only the 3 zones
	std::cout<<"Saving using saveZones()..."<<std::endl;
	dt.saveZones("ex11_saveZones.fade",vSaveZones);

	// * 5 *   Save only one zone
	std::cout<<"Saving using Zone2::save()..."<<std::endl;
	pDiffZone->save("ex11_marsZone.fade");


	// * 6 *   Choose one of the 3 files to reload and show
	std::cout<<std::endl<<"Reloading ex11_save..."<<std::endl;
	Fade_2D loadDt;
	std::vector<Zone2*> vLoadZones;
	bool bOK=loadDt.load("ex11_saveTriangulation.fade",vLoadZones);
	//bool bOK=loadDt.load("ex11_saveZones.fade",vLoadZones);
	//bool bOK=loadDt.load("ex11_marsZone.fade",vLoadZones);
	if(!bOK)
	{
		std::cerr<<"File not loaded, exit"<<std::endl;
		exit(1);
	}

	for(size_t i=0;i<vLoadZones.size();++i)
	{
		Zone2* pZone(vLoadZones[i]);
		std::cout<<"Zone "<<i<<": numT="<<pZone->getNumberOfTriangles()<<std::endl;
		std::string name("loadZone"+toString(i)+".ps");
		pZone->show(name.c_str(),true,true);
	}

	return 0;
}


void createTestZones(Fade_2D& dt,Zone2*& pEarthZone,Zone2*& pMarsZone,Zone2*& pDiffZone)
{
	// Create points for Earth and Mars
	int numPoints(20);
	std::vector<Point2> vEarthPoints;
	std::vector<Point2> vMarsPoints;
	double earthRadius(6371);
	double marsRadius(3389.5);
	generateCircle(numPoints,0,0,earthRadius,earthRadius,vEarthPoints);
	generateCircle(numPoints,5000,0,marsRadius,marsRadius,vMarsPoints);

	// Turn them into polygons and insert as constraints
	std::vector<Segment2> vEarthSegments;
	std::vector<Segment2> vMarsSegments;
	pointsToPolyline(vEarthPoints,true,vEarthSegments);
	pointsToPolyline(vMarsPoints,true,vMarsSegments);
	ConstraintGraph2* pEarthCG(dt.createConstraint(vEarthSegments,CIS_CONSTRAINED_DELAUNAY));
	ConstraintGraph2* pMarsCG(dt.createConstraint(vMarsSegments,CIS_CONSTRAINED_DELAUNAY));
	GASSEX(pEarthCG!=NULL);
	GASSEX(pMarsCG!=NULL);

	// Create two zones and subtract one from the other
	pEarthZone=dt.createZone(pEarthCG,ZL_INSIDE);
	pMarsZone=dt.createZone(pMarsCG,ZL_INSIDE);
	GASSEX(pEarthZone!=NULL);
	GASSEX(pMarsZone!=NULL);
	pDiffZone=zoneDifference(pEarthZone,pMarsZone);

	// Add 4 bounding points
	Bbox2 box(dt.computeBoundingBox());
	std::vector<Point2> vBoxPoints;
	box.getOffsetCorners(500.0,vBoxPoints);
	dt.insert(vBoxPoints);
}
